/**
 * @license elementals.js (c) Jason M. Knight
 * Version 3.6.4 PRODUCTION, 13 April 2018
 * http://www.elementalsjs.com
*/

/*	
	*** MAINTAINER NOTES ***

	do NOT use "this" inside user callable objects like _.Class or _.Node as
	"this" is invalid during certain types of callbacks, such as Array.each
	
	Many of the polyfills are declared as variable functions so we can
	overload them. Remember, static functions cannot be overloaded.
	
	No, the single "=" in some statements below is NOT a typo, they're for
	testing loose value returned as a result WHILE storing that result!
*/

var _;

(function(w, d, ua) {

	"use strict";

	if (!d.head) d.head = d.getElementsByTagName('head')[0];

	var
	
	// *** START local variables ***
	
	blockDisplays = ['block', 'list-item', 'table', 'table-caption', 'table-row'],
	db = d.body,
	de = d.documentElement,
	escMap = {
		'"' : '\\"',
		'\\' : '\\\\',
		'\b' : '\\b',
		'\f' : '\\f',
		'\n' : '\\n',
		'\r' : '\\r',
		'\t' : '\\t'
	},
	escRegex = /[\\\u0000-\u001F\u0022\u2028\u2029]/g,
	htmlSpecialCharsRegex = /[&<>"']/g,
	condenseRegex = /\[\s\xA0]+/g,
	stripCamelRegex = /([A-Z])/g,
	toCamelRegex = /(\W[\w])/g,
	trimRegex = /^[\s\xA0]+|[\s\xA0]+$/g,
	trimLeftRegex = /^[\s\xA0]+/,
	trimRightRegex = /[\s\xA0]+$/,
	ua_msie = ua.match(/MSIE ([0-9\.]+)/i),
	ua_edge = ua.match(/Edge\/([0-9\.]+)/i),
	ua_rv = ua.match(/rv:([0-9\.]+)/i),
	
	// *** END local variables ***
	
	// *** START overwritable functions ***
	
	EventAdd, // defined later
	
	makeSelectionButton = function(targetElement, buttonContent, buttonParent, placement, callback, temp) {
		_.Throw.nonElement(targetElement);
		if (_.NON_CONTENT_TAGS.indexOf(targetElement.tagName.toUpperCase()) !== -1)
			_.Log.Error('Cannot create selection button for <' + targetElement.tagName + '>');
		temp = {
			type : 'button',
			content : buttonContent,
			onclick : callback,
			'data-eNodeSelectTarget' : targetElement
		};
		temp[placement || 'before'] = buttonParent || targetElement;
		_.make('button.selectContents', temp);
	}, // makeSelectionButton	

	// *** END overwritable functions ***
	
	// *** START local objects ***

	iWrite = {
		after : function(e, content) {
			return e.parentNode.insertBefore(domToObject(content), e.nextSibling);
		},
		before : function(e, content) {
			return e.parentNode.insertBefore(domToObject(content), e);
		},
		first : function(e, content) {
			return e.insertBefore(domToObject(content), e.firstChild);
		},
		last : function(e, content) {
			return e.appendChild(domToObject(content));
		},
		rewrite : function(e, content) {
			_.Node.flush(e);
			return _.Node.write(e, content);
		}
	}, // iWrite
	
	/*
		polyfills is an Array of Objects, where each object contains the 
		following properties:
		
			target : Object to apply the polyfill to
		
			prototype : an Object of properties to be applied as target.prototype[]
		
			static : an Object of properties to be applied as target[]
		
		Where properties are out of order, this is done so as to optimize for
		gzip compression. If that's a problem for you, learn to ^F
	*/
	
	polyfills = [{

		target : Array,
		
		prototype : {
		
			callbackAnd : function(a, b) {
				for (var i = 0, l = this.length; i < l; i++)
					if (!b(a, this[i])) return false;
				return true;
			}, // Array.callbackAnd

			callbackAndAll : function(a, b) {
				for (var i = 0, l = this.length, r = false; i < l; i++)
					r = b(a, this[i]) && r;
				return r;
			}, // Array.callbackAndAll

			callbackOr : function(a, b) {
				for (var i = 0, l = this.length, r = false; i < l; i++)
					r = b(a, this[i]) || r;
				return r;
			}, // Array.callbackOr

			every : function(a, b) {
				b = test_funcBVoid(a, b);
				for (var i = 0, l = this.length; i < l; i++)
					if (!a.call(b, this[i], i, this)) return false;
				return true;
			}, // Array.every

			fill : function(a, b, c) {
				tin(this);
				b = b || 0;
				if (b < 0) b += this.length;
				c = undefAssign(c, this.length);
				for (var i = b; i < c; i++) this[i] = a;
				return this;
			}, // Array.fill

			filter : function(a, b) {
				b = test_funcBVoid(a, b);
				for (var r = [], i = 0, e, j = this.length; e = this[i], i < j; i++)
					if (a.call(b, e, i, this)) r.push(e);
				return r;
			}, // Array.filter

			find : function(a, b) {
				b = test_funcBVoid(a, b);
				for (var i = 0, e, l = this.length; e = this[i], i < l; i++)
					if (a.call(b, e, i, this)) return e;
			}, // Array.find

			findIndex : function(a, b) {
				b = test_funcBVoid(a, b);
				for (var i = 0, l = this.length; i < l; i++)
					if (a.call(b, this[i], i, this)) return i;
			}, // Array.findIndex

			forEach : function(a, b) {
				b = test_funcBVoid(a, b);
				for (var i = 0, l = this.length; i < l; i++)
					a.call(b, this[i], i, this);
			}, // Array.forEach

			some : function(a, b) {
				b = test_funcBVoid(a, b);
				for (var i = 0, l = this.length; i < l; i++)
					if (a.call(b, this[i], i, this)) return true;
				return false;
			}, // Array.some
		
			includes : function(a, b) {
				if ((b = test_nullLength(this, b, 0)) === false) return false;
				for (var e, c = a != a, l = this.length; b < l; b++)
					if (a == (e = this[b]) || (c && e != e)) return true;
				return false;
			}, // Array.includes

			indexOf : function(a, b) {
				if ((b = test_nullLength(this, b, 0)) === false) return false;
				for (var c = this.length; b < c; b++) if (this[b] == a) return b;
				return -1;
			}, // Array.indexOf

			lastIndexOf : function(a, b) {
				if ((b = test_nullLength(this, b, this.length)) === false) return false;
				while (b > 0) if (this[--b] == a) return b;
				return -1;
			}, // Array.lastIndexOf

			map : function(a, b) {
				b = test_funcBVoid(a, b);
				for (var i = 0, l = this.length, r = []; i < l; i++)
					r.push(a.call(b, this[i], i, this));
				return r;
			}, // Array.map
			
			reduce : function(a, b) {
				return arrayReducer(a, b, this);
			}, // Array.reduce

			reduceRight : function(a, b) {
				return arrayReducer(a, b, this, true);
			} // Array.reduceRight

		}, // Array prototypes
		
		static : {

			isArray : function(a) {
				return a instanceof Array;
			}, // Array.isArray
			
			'of' : function() {
				return Array.prototype.slice.call(arguments);
			} // Array.of
			
		} // Array statics
			
	}, {
			
		target : Math,
		
		static : {

			// 'constants' 
			RAD : Math.PI / 180,
			QUD : Math.PI / 360,
			DEG : 180 / Math.PI,
			
			// methods
			
			acosh : function(a) {
				return Math.log(a + Math.sqrt(a * a - 1));
			}, // Math.acosh
			
			asinh : function(a) {
				return a === -Infinity ? a : Math.log(a + Math.sqrt(a * a + 1));
			}, // Math.asinh
			
			atanh : function(a) {
				return Math.log((1 + a) / (1 - a)) / 2;
			}, // Math.atanh
			
			cbrt : function(a, b) {
				b = Math.pow(Math.abs(a), 1 / 3);
				return a < 0 ? -b : b;
			}, // Math.cbrt
			
			clz32 : function(a) {
				return !a ? 32 : 31 - Math.floor(Math.log(a >>> 0) * Math.LOG2E);
			},
			
			clz64 : function(a) {
				return !a ? 64 : 63 - Math.floor(Math.log(a >>> 0) * Math.LOG2E);
			},
			
			cosh : function(a) {
				return ((a = Math.exp(a)) + 1 / a) / 2;
			},
			
			/*
				I am NOT polyfilling Math.expm1, type the -1 you lazy ****'s!
				Same for Math.log10, Math.logP, Math.log2
				Why the **** do those even EXIST?!?
				
				Skipping Math.fround, MAYBE later.
			*/
			
			hypot : function() {
				for (var i = 0, j = arguments.length, k, r = 0; i < j; i++) {
					if (
						Infinity == (k = arguments[i]) ||
						Infinity == -k
					) return Infinity;
					r += k * k;
				}
				return Math.sqrt(r);
			}, // Math.hypot
			
			imul : function(a, b) {
				var
					ax = 0xFFFF & a,
					dx = 0xFFFF & b;
					return ax * dx + ((( 
						(0xFFFF & (a >> 16)) * dx +
						(0xFFFF & (b >> 16)) * ax
					) << 16) >>> 0) | 0;
			}, // Meth.imul
			
			sign : function(a) {
				return ((a > 0) - (a < 0)) || +a;
			}, // Math.sign
			
			sinh : function(a) {
				return ((a = Math.exp(a)) - 1 / a) / 2;
			},
			
			tanh : function(a, b, c) {
				return (b = Math.exp(+a)) == Infinity ? 1 : (
					(c = Math.exp(-a)) == Infinity ? -1 : (b - c) / (b + c)
				);
			},
			
			trunc : function (a) {
				return a < 0 ? Math.ceil(a) : Math.floor(a);
			}
		
		} // math Statics
		
	}, {

		target : Number,
		
		static : {
		
			EPSILON : 2.220446049250313e-16,
			MAX_SAFE_INTEGER : 9007199254740991,
			MIN_SAFE_INTEGER : -9007199254740991,
			
			'isNaN' : function(value) {
				return value != value;
			},
			
			'isFinite' : function(value) {
				return 'number' == typeof value && isFinite(value);
			},
			
			isInteger : function(value) {
				return Number.isFinite(value) && Math.floor(value) === value;
			},
			
			isSafeInteger : function(value) {
				return Number.isInteger(value) && Math.abs(value) <= Number.MAX_SAFE_INTEGER;
			}
			
		} // Number statics
		
	}, {

		target : Object,
		
		static : {
		
			keysDoIntersect : function(obj1, obj2, i) {
				for (i in obj1) if (i in obj2) return i;
				return false;
			},
		
			keysIntersection : function(obj1, obj2, i, result) {
				result = [];
				for (i in obj1) if (i in obj2) result.push(i);
				return result;
			}
			
		} // Object statics
		
	}, {

		target : String,
		
		static : {
		
			SAFE_STRING_LIMIT : 0x0FFFFFF0
			
		}, // String statics
		
		prototype : {
		
			condense : function() {
				return this.replace(condenseRegex, ' ').trim();
			}, // String.condense
		
			endsWith : function(a, b) {
				tin(this);
				if (b) _.Throw.isNaN(b);
				if ((!b && b !== 0) || b >= this.length) b = b.length;
				return this.substr(b - a.length, a.length) == a;
			}, // String.endsWith

			htmlSpecialChars : function() {
				return this.replace(htmlSpecialCharsRegex, function(m) {
					return '&#' + m.charCodeAt() + ';';
				});
			},
		
			jsToCssName : function() {
				tin(this);
				var
					v = this.stripCamel(),
					f = this.indexOf('-');
				return ((
					(f > 0) && (f = v.substr(0,f)) &&
					(_.BROWSER_PREFIXES.indexOf(f) !== -1)
				) ? '-' : '') + this;
			}, // String.jsToCssName
			
			pad : function(a, b, c) {
				_.Throw.isNaN(a);
				if ((c = Math.abs(a) - this.length) <= 0) return this;
				b = (!b && b !== 0) ? ' ' : String(b).charAt(0);
				return a < 0 ? b.repeat(c) + this : this + b.repeat(c);
			}, // String.pad
			
			padEnd : function(a, b) {
				return a <= 0 ? this : this.pad(a, b);
			},
			
			padStart : function(a, b) {
				return a <= 0 ? this : this.pad(-a, b);
			},
			
			repeat : function(a) {
				tin(this);
				_.Throw.range(a = a || 0, 0, Math.floor(String.SAFE_STRING_LIMIT / this.length));
				return Array(Math.floor(a) + 1).join(this);
			}, // String.repeat
			
			splitSelector : function(r) {
				_.Throw.isLooseFalse(this);
				r = new selectorStringChunker(this);
				return {
					content : r.next('~'),
					special : r.next('@'),
					classes : r.next('.').replace('.', ' ').trim(),
					id : r.next('#'),
					tag : r.remains.toLowerCase()
				};
			}, // String.splitSelector
			
			startsWith : function(a, b) {
				tin(this);
				if (b) _.Throw.isNaN(b);
				return (a = String(a)) == this.substr(b || 0, a.length);
			}, // String.startsWith

			strFrom : function(a, b) {
				return (b = b ? this.lastIndexOf(a) : this.indexOf(a)) == -1 ? false : this.substr(b + 1);
			}, // String.strFrom

			stripCamel : function(a) {
				tin(this);
				a = undefAssign(a, '-');
				return this.replace(stripCamelRegex, function(m) {
					return a + m.toLowerCase();
				});
			}, // String.stripCamel
			
			toCamel : function(a) {
				tin(this);
				a = undefAssign(a, '-');
				return this.replace(toCamelRegex, function(b, c, e) {
					return (e === 0 ? b : b.toUpperCase()).replace(a, '');
				});
			}, // String.toCamel

			trim : function() {
				tin(this);
				return this.replace(trimRegex, '');
			}, // String.trim

			trimLeft : function() {	
				tin(this);
				return this.replace(trimLeftRegex, '');
			}, // String.trimLeft

			trimRight : function() {
				tin(this);
				return this.replace(trimRightRegex, '');
			} // String.trimRight

		} // String prototypes
		
	}, {
		
		target : w,
		
		static : {
		
			'getComputedStyle' : function(element) {
				return element.currentStyle;
			}
			
		}

	}]; // polyfilla
	
	// *** END local objects ***
	
	// *** START underscore static methods and properties ***
	
	_ = {

		BROWSER_PREFIXES : ['ms', 'moz', 'o', 'webkit'],
		
		Class : {

			add : function(element, cName) {
				return cName instanceof Array ?
					cName.callbackOr(element, _.Class.add) :
					_.Class.exists(element, cName) ? true :
					(element.className = (element.className + ' ' + cName).condense());
			}, // _.Class.add

			exists : function(element, cName) {
				return cName instanceof Array ?
					cName.callbackAnd(element, _.Class.exists) :
					newCNR(cName).test(element.className);
			}, // _.Class.exists

			remove : function(element, cName) {
				if (cName instanceof Array) return cName.callbackOr(element, _.Class.remove);
				if (!_.Class.exists(element, cName)) return false;
				element.className = element.className.replace(newCNR(cName), '').condense();
				return true;
			}, // _.Class.remove

			swap : function(element, oldClass, newClass) {
				oldClass = _.Class.remove(element, oldClass);
				return _.Class.add(element, newClass) || oldClass;
			}, // _.Class.swap

			toggle : function(element, cName) {
				if (_.Class.remove(element, cName)) return false;
				_.Class.add(element, cName);
				return true;
			} // _.Class.toggle

		}, // _.Class

		Compare : {

			string : function(a, b) {
				return _.safeString(a).localeCompare(_.safeString(b));
			},

			stringI : function(a, b) {
				return _.safeString(a, true).localeCompare(_.safeString(b, true));
			},

			numeric : function(a, b) {
				return (Number(a) || 0) - (Number(b) || 0);
			}

		}, // _.Compare

		dataGet : function(element, name, splitResult, temp) {
			return (temp = element.getAttribute(name = 'data-' + name)) === null ? false :
				temp === '' ? element.hasAttribute && element.hasAttribute(name) :
					splitResult ? temp.split(',') : temp;
		}, // _.dataGet

		EMPTY_TAGS : [
			'AREA', 'BASE', 'BR', 'COL', 'COMMAND', 'EMBED', 'HR', 'IMG',
			'INPUT', 'KEYGEN', 'LINK', 'META', 'PARAM', 'SOURCE', 'TRACK',
			'WBR'
		], // _.EMPTY_TAGS
		
		ajax : function(params) {
			switch (typeof params) {
				case 'function':
					params = { 'status' : { '200' : params } };
				case 'object':
					return (new eXMLHttp(params)).result;
			}
			tnt('Invalid Ajax declaration, parameter "handler" not a function or object!');
		}, // _.ajax

		Cookies : {

			'delete' : function(name, path, domain) {
				_.Cookies.get(name) && _.Cookies.set(name, '', 0, path, domain);
			}, // _.Cookies.delete
			
			get : function(name) {
				return name && (name = d.cookie.match(
					new RegExp('(^|;)\\s*' + name + '\\s*=\\s*([^;]*)')
				) ? unescape(name[2]) : null);
			}, // _.Cookies.get

			set : function(name, value, time, path, domain, secure) {
				if (
					'undefined' == typeof time ||
					time === false
				) time = 34158171600000; // 6 June 3052, 10:00AM EST
				if (Number.isInteger(time)) time = new Date(time);
				if (time instanceof Date) time = time.toGMTString();
				_.Throw.nonType(time, 'string');
				d.cookie = name + '=' + escape(value) +
					'; expires=' + time +
					(path ? '; path=' + path : '') +
					(domain ? '; domain=' + domain : '') +
					(secure ? '; secure' : '');
			}, // _.Cookies.set

			setAfter : function(name, value, time, path, domain, secure) {
				_.Cookies.set(name, value, time ? new Date().getTime() + time : false, path, domain, secure);
			} // _.Cookies.setAfter

		}, // _.Cookies

		Event : {

			add : function(e, value, callback, prevent, forceThis) {
				new EventAdd(
					e, value, callback,
					true == prevent ? 1 : (prevent || 0),
					forceThis || null
				);
			},
			
			PREVENT_BUBBLE : 1,
			PREVENT_PROPAGATION : 3

		}, // _.Event

		formSerialize : function(formElement) {

			_.Throw.notTagName(formElement, 'FORM');
			
			var result = '', temp;

			function add(name, value) {
				if (value && ('' !== value)) result +=
					(result ? '&' : '') +
					encodeURIComponent(name) +
					'=' +
					encodeURIComponent(value);
			}

			for (var i = 0, e; e = formElement.elements[i]; i++) {
				if (e.name) switch (e.tagName.toUpperCase()) {
					case 'INPUT':
						switch (e.type) {
							case 'file' :
								// to-do, add serialization for browsers that support it
								tnt('Cannot Serialize a <input type="file">');
							case 'checkbox':
							case 'radio':
								if (!e.checked) continue;
						}
					case 'BUTTON':
					case 'TEXTAREA':
						add(e.name, e.value);
					break;
					case 'SELECT':
						temp = e.options[e.selectedIndex];
						add(e.name, temp.value || _.Node.text(temp));
				}
			}

			return result;

		}, // _.formSerialize

		ids : {},
		
		Load : {

			style : function(href, media) {
				if (media) {
					media = media.toLowerCase();
					if ('all' == media) _.log.warn(
						'media="all" is NOT recommended - ' + href
					);
				}
				return _.make('link', {
					rel : 'stylesheet',
					href : href,
					'media' : media || 'screen,projection,tv',
					last : d.head
				});
			}, // _.Load.style

			script : function(src, attr) {
				return _.make('script@' + src, attr);
			} // _.Load.script

		}, // _.Load

		Log : {

			alert : function(message) {
				alert(message);
			}, // _.Log.alert

			append : function(message) {
				_.Node.write(db, [['br'], message]);
			}, // _.Log.append

			error : function(message) {
				throw new Error(message);
			}, // _.Log.error

			log : function(message) {
				w.console ? console.log(message) : _.Log.append(message);
			}, // _.Log.log

			warn : function(message) {
				w.console ? console.warn(message) : _.Log.append(message);
			} // _.Log.warn

		}, // _.Log
		
		make : function(selector, cmds, e, temp) {
			if (selector instanceof Array) {
				for (var i = 0; e = selector[i]; i++) _.make(e[0], e[1]);
				return true;
			}
			selector = _.Throw.nonStringish(selector);
			if (selector[0] == '~') e = d.createTextNode(selector.substr(1));
			else {
				selector = selector.splitSelector();
				e = d.createElement(selector.tag || 'div');
				if (selector.classes) e.className = selector.classes;
				if (selector.id) _.ids[e.id = selector.id] = e;
				if (selector.special) switch (selector.tag) {
					case 'a' : e.href = selector.special; break;
					case 'button':
					case 'input': e.type = selector.special; break;
					case 'option': e.value = selector.special; break;
					case 'th': e.scope = selector.special; break;
					case 'td': e.headers = selector.special; break;
					case 'script':
						if (!cmds) cmds = {};
						if (!Object.keysDoIntersect(iWrite, cmds)) cmds.last = db;
						// YES, DROP THROUGH!!!
					case 'img': e.src = selector.special;
				}
				if (selector.content) switch (selector.tag) {
					case 'img' : e.alt = selector.content; break;
					case 'input':
					case 'textarea': e.value = selector.content; break;
					default: _.Node.write(e, selector.content);
				}
			}
			if (cmds instanceof Array) return _.Node.write(e, cmds);
			else if ('object' == typeof cmds) {
				var write = false;
				for (var name in cmds) {
					var value = cmds[name];
					if ('on' == name.substr(0, 2)) _.Event.add(e, name.substr(2), value);
					else if (iWrite[name]) {
						if (write) throw new Error(
							'Multiple write methods passed to _.make, only one DOM-JON placement command may be used.'
						);
						write = name;
					} else switch (name) {
						case 'content':
							_.Node.write(e, value);
							break;
						case 'style':
							for (var i in value) e.style[i] = value[i];
							break;
						case 'type':
							// legacy IE can't handle setting type on Element directly
							e.setAttribute(name, value);
							break;
						default:
							// but legacy IE screws up setAttribute a LOT on other cmds so...
							e[name] = value;
					}
				}
				if (write) _.Node.write(cmds[write], e, write);
			} else if (cmds && e.nodeType == 1) iWrite.last(cmds);
			return e;
		}, // _.make

		Metrics : {

			absoluteX : function(element) {
				return element.getBoundingClientRect().left + _.Metrics.Scroll.x();
			}, // _.Metrics.absoluteX

			absoluteY : function(element) {
				return element.getBoundingClientRect().top + _.Metrics.Scroll.y();
			}, // _.Metrics.absoluteY

			height : function(element) {
				return !element || element == w ? (
					w.innerHeight || de.clientHeight || db.clientHeight
				) : element.clientHeight;
			}, // _.Metrics.height

			width : function(element) {
				return !element || element == w ? (
					w.innerWidth || de.clientWidth || db.clientWidth
				) : element.clientWidth;
			}, // _.Metrics.width
			
			Scroll : {

				height : function(element) {
					return (!element || element == w ? db : element).scrollHeight;
				}, // _.Metrics.Scroll.height

				width : function(element) {
					return (!element || element == w ? db : element).scrollWidth;
				}, // _.Metrics.Scroll.width

				maxX : function(element) {
					return _.Metrics.Scroll.width(element) - _.Metrics.width(element);
				}, // _.Metrics.Scroll.maxX

				maxY : function(element) {
					return _.Metrics.Scroll.height(element) - _.Metrics.height(element);
				}, // _.Metrics.Scroll.maxY

				x : function(element) {
					return !element || element == w ? (
						w.scrollX || w.pageXOffset || db.scrollLeft
					) : element.scrollLeft;
				}, // _.Metrics.Scroll.x

				y : function(element) {
					return !element || element == w ? (
						w.scrollY || w.pageYOffset || db.scrollTop
					) : element.scrollTop;
				} // _.Metrics.Scroll.y

			} // _.Metrics.Scroll

		}, // _.Metrics

		Node : {

			addCopyButton : function(
				element, buttonContent, buttonParentElement, placement
			) {
				makeSelectionButton(
					element,
					buttonContent || 'Copy',
					buttonParentElement,
					placement,
					function(a) {
						_.Select.node(a.currentTarget['data-eNodeSelectTarget']);
						_.Select.toClipboard();
					}
				);
			}, // _.Node.addCopyButton

			addSelectButton : function(
				element, buttonContent, buttonParentElement, placement
			) {
				makeSelectionButton(
					element,
					buttonContent || 'Select',
					buttonParentElement,
					placement,
					function(e) {
						_.Select.node(e.currentTarget['data-eNodeSelectTarget']);
					}
				);
			}, // _.Node.addSelectButton

			countAll : function(element, nodeMask) {
				return nCountBy(element, nodeMask, 'eachAll');
			}, // _.Node.countAll

			countChildren : function(element, nodeMask) {
				return nCountBy(element, nodeMask, 'eachChild');
			}, // _.Node.countChildren

			eachAll : function(element, callback, nodeMask) {
				if (element = element.firstChild) do {
					if (
						(_.Node.mask(element, nodeMask) && !callback(element)) ||
						(element.firstChild && !_.Node.eachAll(element, callback, nodeMask))
					) return false;
				} while (element = element.nextSibling);
				return true;
			}, // _.Node.eachAll
			
			eachChild : function(element, callback, nodeMask) {
				if (element = element.firstChild) do {
					if (_.Node.mask(element, nodeMask) && !callback(element)) return false;
				} while (element = element.nextSibling);
				return true;
			}, // _.Node.eachChild
			
			first : function(element, nodeMask) {
				return _.Node.walk(element, nodeMask, 'firstChild', 'nextSibling');
			}, // _.Node.first

			flush : function(element) {
				while (element.firstChild) element.removeChild(element.firstChild);
			}, // _.Node.flush

			last : function(element, nodeMask) {
				return _.Node.walk(element, nodeMask, 'lastChild', 'previousSibling');
			}, // _.Node.last

			mask : function(element, nodeMask) {
				return nodeMask === 0 ? true :
					isNaN(nodeMask = nodeMask || 1) ? (
						element.nodeType == 1 &&
						element.tagName.toUpperCase() == nodeMask.toUpperCase()
					) : ((
						element.nodeType == 0 ? 0 : 1 << (element.nodeType - 1)
					) & nodeMask) > 0;
			}, // _.Node.mask

			next : function(element, nodeMask) {
				return _.Node.walk(element, nodeMask, 'nextSibling', 'nextSibling');
			}, // _.Node.next

			prev : function(element, nodeMask) {
				return _.Node.walk(element, nodeMask, 'previousSibling', 'previousSibling');
			}, // _.Node.prev
			
			// reuse nodeMask in next/prev remove, saves a few bytes in minified/gz
			nextRemove : function(element, nodeMask) {
				nodeMask = _.Node.next(element, nodeMask);
				_.Node.remove(element);
				return nodeMask;
			}, // _.Node.nextRemove

			prevRemove : function(element, nodeMask) {
				nodeMask = _.Node.prev(element, nodeMask);
				_.Node.remove(element);
				return nodeMask;
			}, // _.Node.prevRemove

			remove : function(element) {
				element.parentNode.removeChild(element);
			}, // _.Node.remove

			replace : function(oldNode, newNode) {
				return oldNode.parentNode.replaceChild(domToObject(newNode));
			}, // _.Node.replace
			
			/*
				Why not innerText or textContent for _.Node.text? HERE'S WHY:
				http://perfectionkills.com/the-poor-misunderstood-innerText/
				Hardcode this s*** so it's CONSISTENT! Not rocket science.
			*/
			
			text : function(element) {
				var result = '';
				_.Node.eachAll(element, function(node) {
					result += node.nodeValue + ' '; 
					return true; 
				}, _.Node.mask.TEXT);
				return result.condense();
			},
			
			textFormatted : function(element) {
				var
					result = '',
					blockAfter = false,
					etn,
					style;
				_.Node.eachAll(element, function(e) {
					switch (e.nodeType) {
						case 1:
							style = getComputedStyle(e);
							if (
								style.visibility == 'hidden' ||
								style.display == 'none'
							) break;
							etn = e.tagName.toUpperCase();
							if (
								etn == 'INPUT' ||
								etn == 'TEXTAREA'
							) {
								result += e.nodeValue.condense() + ' ';
								break;
							}
							if (
								etn == 'BR' ||
								blockDisplays.includes(style.display)
							) {
								if (blockAfter) result = result.trimRight() + "\r\n";
								blockAfter = true;
							}
							break;
						case 3:
							result += e.nodeValue.condense() + ' ';
					}
					return true;
				}, 0);
				return result.trim();
			}, // _.Node.textFormatted
			
			walk : function(element, nodeMask, firstMethod, nextMethod, temp) {
				if (temp = element[firstMethod]) do {
					if (_.Node.mask(temp, nodeMask)) return temp;
				} while (temp = temp[nextMethod]);
				return false;
			}, // RE... SPECT... WALK... Whaddaya Say?!?
			
			write : function(element, content, placement) {
				if (!placement || !iWrite[placement]) placement = 'last';
				if (content instanceof Array) {
					for (var i = 0, child; child = content[i]; i++) {
						if (child instanceof Array) {
							if (
								child[1] instanceof Array || (
									'undefined' != typeof child[1] &&
									"object" != typeof child[1]
								)
							) child[1] = { content : child[1] };
							if (!child[1]) child[1] = { last : element };
							else if (!Object.keysDoIntersect(iWrite, child[1])) child[1].last = element;
							child = _.make(child[0], child[1]);
							if (placement != 'last') {
								if (placement != 'after') placement = 'after';
								element = child;
							}
						} else iWrite[placement](element, child);
					}
				} else iWrite[placement](element, content);
			} // _.Node.write
			
		}, // _.Node
		
		safeString : function(stringish, toLowerCase) {
			stringish = (
				'undefined' == typeof stringish ||
				(!stringish && stringish !== 0)
			) ? '' : String(stringish).trim();
			return toLowerCase ? stringish.tolowercase() : stringish;
		}, // _.safeString

		Select : {

			all : function() {
				_.Select.node(db);
			} // _.Select.all

		}, // _.Select

		Throw : {

			isLooseFalse : function(value) {
				if (!value) tnt(value + ' should not be loose false.');
			}, // _.Throw.isLooseFalse

			isNaN : function(value) {
				tin(value);
				if (value != value) tnr('"value" must be a number');
			}, // _.Throw.isNaN

			isNull : function(value) {
				if (null == value || 'undefined' == typeof value) tnt(
					'Variable Null or Undefined'
				);
			}, // _.Throw.isNull
			
			nonElement : function(element) {
				tin(element);
				if (element.nodeType !== 1) tnt(
					element + ' is not a DOM Element'
				);
			}, // _.Throw.nonElement

			nonInstance : function(object1, object2) {
				tin(object1);
				if (!(object1 instanceof object2)) tnt(
					object1 + ' is not an instance of ' + object2
				);
			}, // _.Throw.nonInstance
			
			nonStringish : function(value) {
				if (
					!value ||
					!isNaN(value) ||
					'object' == typeof value ||
					'function' == typeof value ||
					value instanceof Array
				) tnt('"' + value + '" is not a non-empty non-numeric String');
				return String(value);
			},

			nonType : function(value, type) {
				tin(value);
				if (type !== typeof value) tnt(
					value + ' is not a "' + type + '" type'
				);
			}, // _.Throw.nonType

			notTagName : function(element, tagName) {
				tin(element);
				tagName = tagName.toUpperCase();
				if (
					1 != element.nodeType ||
					element.tagName.toUpperCase() != tagName
				) tnt(element + ' is not a <' + tagName + '>');
			}, // _.Throw.notTagName

			range : function(value, min, max) {
				_.Throw.isNaN(value);
				if (value == Infinity) tnr('"value" must not be Inifinty');
				if (!isNaN(min) && value < min) tnr('"value" must be  >= ' + min);
				if (!isNaN(max) && value > max) tnr('"value" must be  <= ' + max);
			}, // _.Throw.range
			
			'type' : function(value, type) {
				if (type == typeof value) tnt(value + ' cannot be type "' + type + '"');
			} // _.Throw.type

		}, // _.Throw
		
		USort : {

			element : function(e, callback) {
				sort = [];
				switch (e.tagName.toUpperCase()) {
					case 'TBODY' :
						_.Node.eachChild(e, function(tr, result) {
							notSetSet(tr, 'data-USortOriginalPlace', sort.length);
							result = [];
							_.Node.eachChild(tr, function(a) {
								return result.push(_.Node.text(a)), true;
							});
							sort.push({ tag : tr.cloneNode(true), data : result });
							return true;
						}, 'TR');
						break;
					case 'UL':
					case 'OL':
						_.Node.eachChild(e, function(li) {
							notSetSet(li, 'data-USortOriginalPlace', sort.length);
							sort.push({ tag : li.cloneNode(true), data : _.Node.text(li) });
							return true;
						}, 'LI');
						break;
					default: tnt(e + ' is not a TBODY, UL, or OL');
				}
				e['data-USortElementSorted'] = true;
				sort.sort(callback);
				_.Node.flush(e);
				for (var i = 0, j; j = sort[i]; i++) e.appendChild(j.tag);
			}, // _.USort.element

			restore : function(e) {
				switch (e.tagName.toUpperCase()) {
					case 'TBODY':
					case 'UL':
					case 'OL':
						if (e['data-USortElementSorted']) _.USort.element(e, originalSort);
						return;
				}
				tnt(e + ' is not a TBODY, UL, or OL');
			} // _.USort.restore

		} // _.USort

	}; // _
	
	// *** END underscore static methods and properties ***

	// internal references to static underscore methods
	// reduces compressed size for some methods
	var tin = _.Throw.isNull;
	
	/*
		*** START local functions ***

		These functions are invisible outside of this self-instancing
		function and are meant for internal use by elementals.js ONLY.
	*/
	
	function browserFix(target, methodName, altMethodName) {
		if (target[methodName]) return false;
		if (!altMethodName) altMethodName =  methodName.charAt(0).toUpperCase() + methodName.substr(1);
		for (var i = 0, j, k; j = _.BROWSER_PREFIXES[i]; i++)
			if (target[k = j + altMethodName]) return target[methodName] = target[k], true;
		return true;
	} // browserFix
	
	function domToObject(value) {
		return 'object' == typeof value ? value : d.createTextNode(value);
	} // domToObject
	
	function eXMLHttp(params, ajax) {

		if (!(ajax = this.result = w.XMLHttpRequest ? new XMLHttpRequest() : (
			newAX('Msxml2.XMLHTTP.6.0') ||
			newAX('Msxml2.XMLHTTP.3.0') ||
			newAX('Microsoft.XMLHTTP')
		))) throw new Error('Unable to create XMLHttpRequest or ActiveX XMLHTTP');

		if (params.progress) ajax.onprogress = params.progress;

		ajax.getJSON = function() {
			return JSON.parse(this.responseText);
		};

		ajax.getXMLValueById = function(id) {
			return (id = this.responseXML.getElementById(id)) ? _.Node.text(id) : null;
		};

		ajax.getXMLValuesByTagName = function(tagName, limit, matches) {
			if (!(matches = this.responseXML.getElementsByTagName(tagName))) return false;
			if (true === limit) return _.Node.text(matches[0]);
			if (!limit || matches.length < limit) limit = matches.length;
			var r = [];
			for (var i = 0; i < limit; i++) r.push(_.Node.text(matches[i]));
			return r;
		};

		ajax.onreadystatechange  = function() {
			if (
				params.state &&
				params.state[ajax.readyState]
			) return params.state[ajax.readyState](ajax, params);
			if (4 == ajax.readyState) {
				if (params.status) {
					if (params.status[ajax.status]) return params.status[ajax.status](ajax, params);
					if (params.status.catchAll) return params.status.catchAll(ajax, params);
				}
				_.Log[params.onStatusLog || 'warn']('Unhandled AJAX Status ' + ajax.status);
			} else if (params.onReadyLog) _.Log[params.onReadyLog]('Unhandled AJAX readyState ' + ajax.readyState);
		};

		if (params.request) {
			ajax.open(params.request[0], params.request[1]);
			if (params.mimeType) ajax.overrideMimeType(params.mimeType);
			if (params.headers)
				for (var i in params.headers)
					ajax.setRequestHeader(i, params.headers[i]);
			ajax.send(params.request[2]);
		}

	} // eXMLHttp

	function newAX(axName) {
		try { return new activeXObject(axName); } catch (axName) { return false; }
	} // newAX
	
	function newCNR(cName) {
		return new RegExp('(\\s|^)' + cName + '(\\s|$)');
	}

	function nCountBy(element, nodeMask, method, i) {
		i = 0;
		_.Node[method](element, function() { return i++; }, nodeMask);
		return i;
	} // nCountBy

	function originalSort(a, b) {
		return _.Compare.numeric(
			a.tag.getAttribute('data-USortOriginalPlace'),
			b.tag.getAttribute('data-USortOriginalPlace')
		);
	} // originalSort

	function resultArray() {
		this.result = [];
		this.add = function(v) { this.result.push(v); }
	}
	
	function selectorStringChunker(str) {
		this.remains = str;
		this.next = function(match, i, r) {
			if ((i = this.remains.indexOf(match)) == -1) return '';
			r = this.remains.substr(i + match.length);
			this.remains = this.remains.substr(0, i);
			return r.length == 0 ? false : r;
		}
	}
	
	function test_funcBVoid(a, b) {
		_.Throw.nonType(a, 'function');
		return 'undefined' == typeof b ? void 0 : b;
	}
	
	function test_nullLength(arr, b, value) {
		tin(arr);
		return arr.length == 0 ? false : (
			'undefined' == typeof b || (b < 0 && (b += this.length) < 0)
		) ? value : b;
	}
	
	function tnt(message) { throw new typeError(message); }
	function tnr(message) { throw new rangeError(message); }
	
	function notSetSet(e, attrName, value) {
		if (!e.getAttribute(attrName)) e.setAttribute(attrName, value);
	}
	
	function undefAssign(e, fallback) {
		return (undefined == typeof e) ? fallback : e;
	}
	
	// *** END local functions ***
	
	iWrite.replace = _.Node.replace;
	
	// *** START static and generated underscore properties ***

	_.IS_IE = ua_msie ? ua_msie[1] : (/Trident/i.test(ua) ? ua_rv[1] : false);
	_.IS_EDGE = (ua_edge && !_.IS_IE) ? ua_edge[1] : false;

	_.Node.mask.ALL        = 0x0000;
	_.Node.mask.ELEMENT    = 0x0001;
	_.Node.mask.TEXT       = 0x0004;
	_.Node.mask.PROCESSING = 0x0040;
	_.Node.mask.COMMENT    = 0x0080;
	_.Node.mask.DOCUMENT   = 0x0100;
	_.Node.mask.DOCTYPE    = 0x0200;
	_.Node.mask.FRAGMENT   = 0x0400;
	
	_.NON_CONTENT_TAGS = _.EMPTY_TAGS.concat([
		'AUDIO', 'COLGROUP', 'HEAD', 'MAP', 'METER', 'PROGRESS',
		'OBJECT', 'SCRIPT', 'STYLE', 'TEXTAREA', 'VIDEO'
	]); // _.NON_CONTENT_TAGS

	// *** END static and generated underscore properties ***
	
	// *** START browser specific undersscore methods ***
	
	// START browser specific _.Event.prevent and local EventAdd object
	if (w.addEventListener) {
		EventAdd = function(element, event, callback, captureLevel, forceThis) {
			element.addEventListener(event, function(e) {
				callback.call(forceThis, e);
				if (captureLevel > 1) _.Event.prevent(e);
			}, captureLevel == 1);
		};
		_.Event.prevent = function(e) {
			if (e.stopPropagation) e.stopPropagation();
			if (e.preventDefault) e.preventDefault();
		};
	} else {
		_.Event.prevent = function(e) {
			e.cancelBubble = true;
			e.returnValue = false;
		};
		EventAdd = function(element, event, callback, captureLevel, forceThis) {
			element.attachEvent('on' + event, function(e) {
				e = w.event;
				e.target = w.srcElement;
				e.currentTarget = element;
				callback.call(forceThis, e);
				if (captureLevel > 0) {
					e.cancelBubble = true;
					if (captureLevel > 1) e.returnValue = false;
				}
			});
		};
	}
	// END browser specific _.Event.prevent and local EventAdd object
	
	// START browser specific _.Select.node and _.Select.none
	if (d.selection) {
		_.Select.node = function(element, temp) {
			temp = db.createTextRange();
			temp.moveToElementText(element);
			temp.select();
			return true;
		};
		_.Select.none = function() {
			d.selection.empty();
			return true;
		};
	} else if (w.getSelection) {
		_.Select.node = function(element, tempRange, tempSel) {
			tempRange = d.createRange();
			tempRange.selectNodeContents(element);
			tempSel = w.getSelection();
			tempSel.removeAllRanges();
			tempSel.addRange(tempRange);
			return true;
		};
		_.Select.none = function() {
			w.getSelection().removeAllRanges();
			return true;
		};
	} else {
		_.Select.node =
		makeSelectionButton = function() {
			_.Log.warn('selection of content not supported');
			return false;
		};
		_.Select.none = function() {
			_.Log.warn('de-selection of content not supported');
			return false;
		};
	}
	// END browser specific _.Select.node and _.Select.none

	// START browser specific _.writeToClipboard, _.Select.toClipBoard
	if (
		w.clipboardData &&
		w.clipboardData.setData
	) {
		_.writeToClipboard = function(a) {
			w.clipboardData.setData('Text', String(a));
			return true;
		};
		_.Select.toClipboard = function() {
			_.writeToClipboard(
				'function' == typeof d.getSelection ?
				String(d.getSelection()) :
				d.selection.createRange().text
			);
			return true;
		};
	} else if (d.queryCommandSupported('copy')) {
		_.writeToClipboard = function(a, b) {
			b = _.make('textarea', { content : String(a), last : db });
			b.select();
			d.execCommand('copy');
			_.Node.remove(b);
			return true;
		};
		_.Select.toClipboard = function() {
			d.execCommand('copy');
			return true;
		};
	} else {
		_.writeToClipboard =
		_.Select.toClipboard = function() {
			_.Log.warn('This browser does not allow JavaScript to access the clipboard');
			return false;
		};
	}
	// END browser specific _.writeToClipboard, _.Select.toClipBoard

	// START browser specific _.Query and _.Query.all
	if ('function' == typeof d.querySelectorAll) {
		_.Query = function(element, selector) {
			return element.querySelector(selector);
		};
		_.Query.all = function(element, selector) {
			// [].slice makes it return an ARRAY
			return [].slice.call(element.querySelectorAll(selector));
		};
	} else {
		var tempStyle = d.createStyleSheet();
		_.Query = function(element, selector, temp) {
			temp = _.Query.all(element, selector, 1);
			return (temp && temp.length > 0) ? temp[0] : null;
		};
		_.Query.all = function(element, selector, limit, results) {
			if (!limit) limit = 32767;
			results = [];
			tempStyle.addRule(selector, 'fu:bar');
			function callback(e) {
				if ((limit > 0) && (e.currentStyle.fu === 'bar')) results.push(e);
				return --limit > 0;
			}
			_.Node.eachAll(element, callback);
			tempStyle.removeRule();
			return results;
		};
	}
	// END browser specific _.Query and _.Query.all

	// *** END browser specific undersscore methods ***

	// *** START polyfills and browser object extensions ***
	
	for (var i = 0, set; set = polyfills[i]; i++) {
		if (set.prototype)
			for (var name in set.prototype)
				if (!set.target.prototype[name])
					set.target.prototype[name] = set.prototype[name];
		if (set.static)
			for (var name in set.static)
				if (!set.target[name])
					set.target[name] = set.static[name];
	}

	// START JSON polyfill
	if (_.POLYFILL_JSON = !w.JSON) w.JSON = {
		parse : function(value) {
			_.Log.warn('Possibly insecure JSON polyfill call');
			return eval('(' + value + ')');
		},
		stringify : function(value, temp) {
			if (null == value) return null;
			if (!isNaN(value)) return isFinite(value) ? value.toString() : 'null';
			switch (typeof value) {
				case 'boolean': return value.toString();
				case 'object':
					if ('function' == typeof value.toJSON) return JSON.stringify(value.toJSON());
					temp = [];
					if (value instanceof Array) {
						for (var i = 0, j = value.length; i < j; i++)
							temp.push(JSON.stringify(value[i]));
						return '[' + temp.join() + ']';
					}
					for (var i in value)
						temp.push(JSON.stringify(i) + ':' + JSON.stringify(value[i]));
					return '{' + temp.join() + '}';
			}
			return '"' + value.toString().replace(escRegex, function(c) {
				return escMap[c] || '\\u' + (c.charCodeAt(0) + 65536).toString(16).substr(1);
			}) + '"';
		}
	};
	// END JSON Polyfill
	
	/*
		START performance and requestAnimation Polyfill
		Based on code by Erik Möller. I'd link but the bloody page is gone!
		Thanks "My Opera"
	*/
	var
		perfLastTime = 0,
		perf = false == 'performance' in w ? w.performance = {} : w.performance;

	if (false == 'now' in perf) {
		var nowOffset = (
			perf.timing && perf.timing.navigationStart
		) ? perf.timing.navigationStart : new Date().getTime();
		perf.now = function() { return new Date().getTime() - nowOffset; };
	}

	if (
		browserFix(w, 'requestAnimationFrame')
	) w.requestAnimationFrame = function(callback, now, elapsed) {
		perfLastTime = (
			elapsed = Math.max(0, 50/3 - (now = perf.now()) - perfLastTime)
		) + now;
		return setTimeout( function() { callback(perfLastTime); }, elapsed);
	};

	if (
		browserFix(w, 'cancelAnimationFrame') &&
		browserFix(w, 'cancelAnimationFrame', 'CancelRequestAnimationFrame')
	) w.cancelAnimationFrame = clearTimeout;
	
	// END performance and requestAnimation Polyfill

	// *** END polyfills and browser object extensions ***

	/*
		Add 'elementals' class to body to say "yes, elementals is running
		handy if we need it as a CSS or other such trigger.
	*/
	_.Class.add(db, 'elementals');
	
	// create existing DOM Id's in _.ids if desired
	if (_.dataGet(db, 'elementals-idParse'))
		_.Node.eachAll(d, function(e) { 
			if (e.id) _.ids[e.id] = e;
			return true;
		});
	
})(window, document, navigator.userAgent);